function [pitch, time, amp] = getpitch(y,fs)
% [pitch, time, amp] = getpitch(y,fs)
%  
%      This function takes a vector of linearly encoded PCM values and
%      returns a pitch track of them, rounded to the nearest MIDI pitch number. 
%      If there is no pitch at a given time step the pitch number returned will 
%      be 0. 
%  
%      Example usage:  
%         load mtlb;
%         [pitch, time, amp] = getpitch(mtlb,Fs);
%  
%  
%   Output values
%   ----------------
%   pitch         - a vector of values where pitch(i) is the best guess
%                   for the MIDI pitch at time t(i)
%  
%   time          - a vector of values where t(i) is the time at which
%                   pitch(i)is determined (measured in seconds)
% 
%   amp           - a vector of amplitudes, where amp(i) is the amplitude of
%                   the signal at time(i)
%  
%   Input arguments 
%   ---------------------------------------------------------
%   y                   - a vector of values assumed to be linearly encoded, 
%                         PCM audio
%   fs                  - a scalar that specifies sample frequency of the 
%                         input audio                         
% 
%   Author: Bryan Pardo
%   Email: pardo@northwestern.edu
%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define some bins for MIDI notes
keyfreq = [-100:-1];
keyfreq(33) = 55;
keyfreq(21) = 55/2;
keyfreq(9) = 55/4;

for i = 10:100 
    keyfreq(i) = keyfreq(9) * 2^((i-9)/12);
end


% do the pitch tracking 

[frq_path, ac_path, time, amp] = praat_pd(y,fs);

% find all  zero frequency values and make them a small positive value
idx = find(frq_path == 0);
frq_path(idx)= 1;

% convert frq_path in Hz to MIDI key numbers
key_path = zeros(size(frq_path));
for i = 1: length(frq_path)
    key_path(i) = 12*log2(frq_path(i)/keyfreq(9)) + 9;
end

idx = find(key_path < 0);
key_path(idx) = 0;

pitch = key_path;

