function notematrix = quantize(nm, pickupBool);
%Quantize generates a quantized matrix for use with the midi toolbox
%Supports 16th, 12th, 8th, 4th, and above notes
%
%If pickupBool is not set (to 1), then any empty starting notes will be
%deleted. To prevent this, set pickupBool to 1.
%
%The following are various cost measures
%tripRestCost  = cost that a triplet begins a rest. percentage of value
%lateNoteCost  = assumes notes are played early. portion of a beat
%longNote      = assumes notes are played shorter. portion of a beat
%stdTriplet    = weight that triplets are in groups of 3. ratio
notematrix = nm;

%Long Note Cost is used since it is more often for a note to be cut short
%than held longer.
%trip rest cost is used in case there is a rest at the start of a triplet.
%This is unlikely.


if nargin == 1
    pickupBool = 0;
end


%These values may be changed to suit the music
tripRestCost  = .50;
lateNoteCost  = .1;
longNoteCost  = .15;
stdTriplet    = .5;


%set first onset time to 0 and all other onsets moved back. this is an
%assumption of the music. this is set in the pickupBool tag.
if pickupBool == 0
    offset = notematrix(1,1);
    notematrix(:,1) = notematrix(:,1)-offset;
end


%extract onsets and durations
onsets   = notematrix(:,1);
durs     = notematrix(:,2);
durMain  = fix(durs);
durSub   = mod(durs,1);
onMain   = fix(onsets);
onSub    = mod(onsets,1); 

refNotes = [0 1/2 1/3 1/4 2/3 3/4 1];
num   = 1:length(refNotes);

%Quantize the onsets and duration
for i = 1:size(notematrix,1)
    qDur = 0; qOn = 0;
    durDist = durSub(i)-refNotes;
    onDist = onSub(i)-refNotes;
    
    %add longNoteCost to all distances that would make the note longer
    %add earlyNoteCost to all distances less than 0
    for j = 1:size(durDist,2)
        durDist(j) = durDist(j)+(longNoteCost*(durDist(j)>0));
        onDist(j)  = onDist(j)+(lateNoteCost*(onDist(j)<0));
    end

    %if the last 1 or 2 notes were triplets, then this one probably is too.
    if i > 1 & notematrix(i-1,2) == 1/3  
        if i > 3 & notematrix(i-3,2) ~= 1/3
            durDist(3) = durDist(3)*stdTriplet;
        else
            durDist(3) = durDist(3)*stdTriplet;
        end
    end 
    
    minDurDist = min(abs(durDist));
    minOnDist  = min(abs(onDist));
   
    qDur = find(abs(durDist)==minDurDist);
    qOn  = find(abs(onDist)==minOnDist);
    
    %This changes the onset time if it is on 1/3 or 2/3 beat
    %with a non-triplet note before, since this is rare to happen.
    if qOn == 3 | qOn == 5
        prevDur = notematrix(i-1,2);
        if prevDur ~= 1/3 & prevDur ~= 2/3
            onDist(3) = onDist(3)+((-1*(onDist(3)<0))+(onDist(3)*tripRestCost));
            onDist(5) = onDist(5)+((-1*(onDist(5)<0))+(onDist(5)*tripRestCost));
            minOnDist = min(abs(onDist));
            qOn = find(abs(onDist)==minOnDist);
        end
    end
    
    %quantized durations and onsets.
    newDur = durMain(i) + refNotes(qDur);
    newOn  = onMain(i)  + refNotes(qOn);
    
    %Make sure the next note is after the previous note.
    if i>1 & sum(notematrix(i-1,1:2)) > newOn
        newOn = sum(notematrix(i-1,1:2));
    end
    
    notematrix(i,1:2) = [newOn newDur];
end